using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Microsoft.Ink;
using System.Diagnostics;
using System.Xml;

namespace JNTReader {
    public partial class MainForm : Form {
        public MainForm() {
            InitializeComponent();
        }

        private void OpenMenuItem_Click( object sender, EventArgs e ) {
            Stream jntStream = null;
            Stream xmlStream = null;
            OpenFileDialog openFileDialog1 = new OpenFileDialog();

            // Get the filename from the user by using a File Open dialog
            openFileDialog1.InitialDirectory = "c:\\";
            openFileDialog1.Filter =
              "Journal files (*.jnt)|*.jnt|All files (*.*)|*.*";
            openFileDialog1.FilterIndex = 1;
            openFileDialog1.RestoreDirectory = true;

            if( openFileDialog1.ShowDialog() == DialogResult.OK ) {
                try {
                    if( (jntStream = openFileDialog1.OpenFile()) != null ) {
                        // Read in the Journal file
                        xmlStream = JournalReader.ReadFromStream( jntStream );

                        // Display results
                        DisplayXml( xmlStream );
                    }
                } catch( System.Exception ex ) {
                    Debug.WriteLine( ex.Message );
                } finally {
                    // Clean up
                    if( xmlStream != null )
                        xmlStream.Close();
                    if( jntStream != null )
                        jntStream.Close();
                }
            }
        }

        #region XML

        // Reads in the XML from the stream
        public void DisplayXml( Stream xmlStream ) {
            System.Xml.XmlDocument xmlDoc = new System.Xml.XmlDocument();
            xmlDoc.Load( xmlStream );
            this.MainTextBox.Text = ReadDoc( xmlDoc );
        }

        // Converts the XmlDocument into human readable text
        public string ReadDoc( XmlDocument doc ) {
            string output = "";
            try {
                output += WriteXml( doc );
            } catch( Exception ) {
            }
            return output;
        }

        // Recursive function for displaying Xml Nodes
        private string WriteChildNode( System.Xml.XmlNode node, int depth ) {
            string output = "";

            // Write out this Node
            if( node.NodeType == XmlNodeType.Element ) {
                output += indent( depth * 4 );
                output += "<" + node.Name;
                foreach( XmlAttribute a in node.Attributes )
                    output += " " + a.Name + "=\"" + a.Value + "\" ";

                if( node.ChildNodes.Count == 0 ) {
                    output += "/>" + System.Environment.NewLine;
                } else {
                    output += ">" + System.Environment.NewLine;
                }

            } else if( node.NodeType == XmlNodeType.Text ) {
                if( node.Value.Length != 0 ) {
                    output += indent( depth * 4 );
                    string data = node.Value.Replace( System.Environment.NewLine, System.Environment.NewLine + indent( depth * 4 ) );
                    output += data + System.Environment.NewLine;
                }
            }

            // Write Child Nodes
            if( node.ChildNodes.Count != 0 ) {
                foreach( XmlNode child in node.ChildNodes ) {
                    output += WriteChildNode( child, depth + 1 );
                }

                output += indent( depth * 4 );
                output += "</" + node.Name + ">" + System.Environment.NewLine;
            }

            return output;
        }

        // Base function for displaying a root node and all child nodes
        private string WriteXml( System.Xml.XmlDocument xmlDoc ) {
            XmlNode xmlNode = xmlDoc.CloneNode( true );

            return WriteChildNode( xmlNode, -1 );
        }

        // Creates a string of spaces with length of 'number'
        private string indent( int number ) {
            string spaces = "";
            for( int i = 0; i < number; i++ ) {
                spaces += " ";
            }
            return spaces;
        }

        #endregion
    }
}